import { SSEClient } from '../sseClient';

// Mock EventSourcePolyfill
jest.mock('event-source-polyfill', () => {
  return {
    EventSourcePolyfill: jest.fn().mockImplementation(() => {
      return {
        addEventListener: jest.fn(),
        close: jest.fn(),
        onopen: null,
        onerror: null
      };
    })
  };
});

describe('SSEClient', () => {
  beforeEach(() => {
    // Reset mocks
    jest.clearAllMocks();
    
    // Set debug mode
    (globalThis as any).__SSE_DEBUG_ENABLED = false;
  });
  
  it('should create a new SSE client', () => {
    const client = new SSEClient('https://example.com/events');
    expect(client).toBeDefined();
  });
  
  it('should connect to the SSE endpoint', () => {
    const client = new SSEClient('https://example.com/events');
    client.connect();
    
    // EventSourcePolyfill should be called with the URL and options
    expect(require('event-source-polyfill').EventSourcePolyfill).toHaveBeenCalledWith(
      'https://example.com/events',
      expect.objectContaining({
        headers: {},
        withCredentials: true
      })
    );
  });
  
  it('should add event listeners', () => {
    const client = new SSEClient('https://example.com/events');
    const listener = jest.fn();
    
    client.on('message', listener);
    client.connect();
    
    // EventSourcePolyfill.addEventListener should be called with the event name and a function
    expect(require('event-source-polyfill').EventSourcePolyfill().addEventListener)
      .toHaveBeenCalledWith('message', expect.any(Function));
  });
  
  it('should close the connection', () => {
    const client = new SSEClient('https://example.com/events');
    client.connect();
    client.close();
    
    // EventSourcePolyfill.close should be called
    expect(require('event-source-polyfill').EventSourcePolyfill().close).toHaveBeenCalled();
  });
  
  it('should handle connection errors', () => {
    const client = new SSEClient('https://example.com/events', {
      autoReconnect: false
    });
    client.connect();
    
    // Get the error handler
    const errorHandler = require('event-source-polyfill').EventSourcePolyfill().onerror;
    
    // Simulate an error
    if (errorHandler) {
      errorHandler(new Error('Connection error'));
    }
    
    // No reconnection should be attempted (autoReconnect is false)
    expect(require('event-source-polyfill').EventSourcePolyfill).toHaveBeenCalledTimes(1);
  });
});
